import logging
from pathlib import Path

from bbot.errors import *

log = logging.getLogger("bbot.presets.path")

DEFAULT_PRESET_PATH = Path(__file__).parent.parent.parent / "presets"


class PresetPath:
    """
    Keeps track of where to look for preset .yaml files
    """

    def __init__(self):
        self.paths = [DEFAULT_PRESET_PATH]

    def find(self, filename):
        filename_path = Path(filename).resolve()
        extension = filename_path.suffix.lower()
        file_candidates = set()
        extension_candidates = {".yaml", ".yml"}
        if extension:
            extension_candidates.add(extension.lower())
        else:
            file_candidates.add(filename_path.stem)
        for ext in extension_candidates:
            file_candidates.add(f"{filename_path.stem}{ext}")
        file_candidates = sorted(file_candidates)
        file_candidates_str = ",".join([str(s) for s in file_candidates])
        paths_to_search = self.paths
        if "/" in str(filename):
            if filename_path.parent not in paths_to_search:
                paths_to_search.append(filename_path.parent)
        log.debug(f"Searching for preset in {paths_to_search}, file candidates: {file_candidates_str}")
        for path in paths_to_search:
            for candidate in file_candidates:
                for file in path.rglob(candidate):
                    log.verbose(f'Found preset matching "{filename}" at {file}')
                    self.add_path(file.parent)
                    return file.resolve()
        raise ValidationError(
            f'Could not find preset at "{filename}" - file does not exist. Use -lp to list available presets'
        )

    def __str__(self):
        return ":".join([str(s) for s in self.paths])

    def add_path(self, path):
        path = Path(path).resolve()
        if path in self.paths:
            return
        if any(path.is_relative_to(p) for p in self.paths):
            return
        if not path.is_dir():
            log.debug(f'Path "{path.resolve()}" is not a directory')
            return
        self.paths.append(path)

    def __iter__(self):
        yield from self.paths


PRESET_PATH = PresetPath()



flag_descriptions = {
    "active": "Makes active connections to target systems",
    "affiliates": "Discovers affiliated hostnames/domains",
    "aggressive": "Generates a large amount of network traffic",
    "baddns": "Runs all modules from the DNS auditing tool BadDNS",
    "cloud-enum": "Enumerates cloud resources",
    "code-enum": "Find public code repositories and search them for secrets etc.",
    "deadly": "Highly aggressive",
    "email-enum": "Enumerates email addresses",
    "iis-shortnames": "Scans for IIS Shortname vulnerability",
    "passive": "Never connects to target systems",
    "portscan": "Discovers open ports",
    "report": "Generates a report at the end of the scan",
    "safe": "Non-intrusive, safe to run",
    "service-enum": "Identifies protocols running on open ports",
    "slow": "May take a long time to complete",
    "social-enum": "Enumerates social media",
    "subdomain-enum": "Enumerates subdomains",
    "subdomain-hijack": "Detects hijackable subdomains",
    "web-basic": "Basic, non-intrusive web scan functionality",
    "web-paramminer": "Discovers HTTP parameters through brute-force",
    "web-screenshots": "Takes screenshots of web pages",
    "web-thorough": "More advanced web scanning functionality",
}



from .base import ModuleTestBase


class TestFullhunt(ModuleTestBase):
    config_overrides = {"modules": {"fullhunt": {"api_key": "asdf"}}}

    async def setup_before_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://fullhunt.io/api/v1/auth/status",
            match_headers={"x-api-key": "asdf"},
            json={
                "message": "",
                "status": 200,
                "user": {
                    "company": "nightwatch",
                    "email": "jonsnow@nightwatch.notreal",
                    "first_name": "Jon",
                    "last_name": "Snow",
                    "plan": "free",
                },
                "user_credits": {
                    "credits_usage": 0,
                    "max_results_per_request": 3000,
                    "remaining_credits": 100,
                    "total_credits_per_month": 100,
                },
            },
        )
        module_test.httpx_mock.add_response(
            url="https://fullhunt.io/api/v1/domain/blacklanternsecurity.com/subdomains",
            match_headers={"x-api-key": "asdf"},
            json={
                "domain": "blacklanternsecurity.com",
                "hosts": [
                    "asdf.blacklanternsecurity.com",
                ],
                "message": "",
                "metadata": {
                    "all_results_count": 11,
                    "available_results_for_user": 11,
                    "domain": "blacklanternsecurity.com",
                    "last_scanned": 1647083421,
                    "max_results_for_user": 3000,
                    "timestamp": 1684541940,
                    "user_plan": "free",
                },
                "status": 200,
            },
        )

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"



#!/usr/bin/env python3
import os
import sys
from pathlib import Path
from Crypto.Cipher import AES
from Crypto.Util.Padding import unpad

ENV_VAR_NAME = "BBOT_SUDO_PASS"
KEY_ENV_VAR_PATH = "BBOT_SUDO_KEYFILE"


def decrypt_password(encrypted_data, key):
    iv, ciphertext = encrypted_data.split(":")
    iv = bytes.fromhex(iv)
    ct = bytes.fromhex(ciphertext)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    pt = unpad(cipher.decrypt(ct), AES.block_size)
    return pt.decode("utf-8")


def main():
    encrypted_password = os.environ.get(ENV_VAR_NAME, "")
    # remove variable from environment once we've got it
    os.environ.pop(ENV_VAR_NAME, None)
    encryption_keypath = Path(os.environ.get(KEY_ENV_VAR_PATH, ""))

    if not encrypted_password or not encryption_keypath.is_file():
        print("Error: Encrypted password or encryption key not found in environment variables.", file=sys.stderr)
        sys.exit(1)

    try:
        key = encryption_keypath.read_bytes()
        decrypted_password = decrypt_password(encrypted_password, key)
        print(decrypted_password, end="")
    except Exception as e:
        print(f'Error decrypting password "{encrypted_password}": {str(e)}', file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



from .base import ModuleTestBase


class TestC99(ModuleTestBase):
    config_overrides = {"modules": {"c99": {"api_key": "asdf"}}}

    async def setup_before_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://api.c99.nl/randomnumber?key=asdf&between=1,100&json",
            json={"success": True, "output": 65},
        )
        module_test.httpx_mock.add_response(
            url="https://api.c99.nl/subdomainfinder?key=asdf&domain=blacklanternsecurity.com&json",
            json={
                "success": True,
                "subdomains": [
                    {"subdomain": "asdf.blacklanternsecurity.com", "ip": "1.2.3.4", "cloudflare": True},
                ],
                "cached": True,
                "cache_time": "2023-05-19 03:13:05",
            },
        )

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"



from bbot.modules.base import BaseModule


class postman(BaseModule):
    """
    A template module for use of the GitHub API
    Inherited by several other github modules.
    """

    base_url = "https://www.postman.com/_api"
    html_url = "https://www.postman.com"
